import sys
import ctypes

import os
import threading
import enum 
import base64
import hashlib
import tempfile
import tarfile
import importlib.util

import memigui as g
from memigui import *


# GUI Manager Callback implementation
class GUIManagerCB(g.GUIManagerCB):
  def run(self, n):
    print('Dummy GUIManagerCB run callback - not implemented - n = ', n)

  def MessageReceived(self, msg):
    #if self.OnGuiMessage != None: 
    #  self.OnGuiMessage(msg)
    #else:
    #print('Message received: ', msg.type, " ", msg.m_evt_type)
    
    mgr.OnGuiMessage(msg)


# CAN Device Callback implementation
class CANDevCB(g.CANDevCB):
  def run(self, n):
    print('Dummy CANDevCB run callback - not implemented - n = ', n)

  def MessageReceived(self, id, data, len):
    byte_str = ctypes.string_at(int(data), len)

    mgr.OnCanMessage(id, byte_str, len)

    #if CanMsgType(id) in [CanMsgType.CAN_ENC1, CanMsgType.CAN_ENC2, CanMsgType.CAN_ENC3, CanMsgType.CAN_ENC4]:
    #  if mgr.OnCanMessage != None: 
    #  
    #  # msg=g.tEM_Message()
    #  # msg.type=0x112
    #  # msg.flag=byte_str[0]
    #  # msg.param=0
    #  # vlblW3.OnMessage(msg)
    #  # #if byte_str[0] == 0x82:
    #  #   #print(f"Weel {id} turn up {byte_str[0]}")
    #  # #else:
    #  #   #print(f"Weel {id} turn down {byte_str[0]}")
    #  # #pass
    #else:
    #  print(f"Unhandled CAN Message received: id {id}")
    #  print("Data: " , byte_str[:len].hex()) 
    #  pass  


class PollThread(threading.Thread):
  def __init__(self, interval, target_func, name=None, *args, **kwargs):
    super(PollThread, self).__init__(name=name)
    self.interval = interval
    self.name = name
    self.target_func = target_func
    self.args = args
    self.kwargs = kwargs
    self.event = threading.Event()

  def run(self):
    while not self.event.wait(self.interval):
      self.target_func(*self.args, **self.kwargs)

  def stop(self):
    self.event.set()

class mpkgmanager():

  dc = None
  cDcb = None
  cD = None
  guicb = None
  gui = None
  pkgr = None
  tPT = None

  def __init__(self, filename) -> None:
    # Set CTheme Colors to be used
    g.CTheme.SetNamedColor("DCSelector", g.CTheme.ColorFromRGB(0, 255, 0))
    g.CTheme.SetNamedColor("MyVLbl3ValLblBorder", g.CTheme.ColorFromRGB(0, 255, 255))
    
    g.CTheme.SetNamedColor("ValLblBg", g.CTheme.ColorFromRGB(32, 32, 32))
    #g.CTheme.SetNamedColor("ValLblBorderAcc", g.CTheme.ColorFromRGB(192, 0, 0))
    g.CTheme.SetNamedColor("ValLblBorderAcc", g.CTheme.ColorFromRGB(90, 156, 162))
    g.CTheme.SetNamedColor("ValLblTitleText", g.CTheme.ColorFromRGB(164, 164, 164))
    g.CTheme.SetNamedColor("ValLblText", g.CTheme.ColorFromRGB(192, 192, 192))
    g.CTheme.SetNamedColor("ValLblTextAcc", g.CTheme.ColorFromRGB(255, 255, 255))
    
    g.CTheme.SetNamedColor("MPMMsgBoxTitleText",0xFFFFFF)
    g.CTheme.SetNamedColor("MPMMsgBoxTitleTextBg",0x5A9CA2)
    g.CTheme.SetNamedColor("MPMMsgBoxBorder",0x5A9CA2)
    g.CTheme.SetNamedColor("MPMMsgBoxTitleBg",0x5A9CA2)

    # Initialize GUI Drawing Context
    self.dc=g.emDC()
    dc=self.dc

    # Initialize CAN Device
    self.cD=g.CCANDev()
    cD = self.cD
    cD.Open("can0")
    
    # Initialize and register CAN callback
    self.cDcb=CANDevCB()
    cDcb=self.cDcb
    cD.registerCallback(cDcb)
    
    # Initialize GUI 
    #gui=GUIManager(dc,cD,"/dev/input/event0")
    self.gui=GUIManager(dc,None,"/dev/input/event0")
    gui = self.gui
    gui.InitGUI()
    
    # Initialize and register GUIManager callback
    self.guicb = GUIManagerCB()
    guicb = self.guicb
    gui.setSendMsgCallback(guicb)

    # Example usage: extract and run from example.tar.gz file
    # extract_and_run( "MD900-Backup-Restore.md900.pkg", tmpDir.name)
    tmpDir = tempfile.TemporaryDirectory()
    
    #self.extract_and_verify(filename, tmpDir.name)
    self.runPkgRunner(filename)

  def runPkgRunner(self, tmp_pkg_path):

    self.pkgPath = tmp_pkg_path
    # Import mpkg module from extracted directory
    spec = importlib.util.spec_from_file_location("mpkg", os.path.join(tmp_pkg_path, "mpkg.py"))
    mpkg = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(mpkg)

    self.pkgr = mpkg.pkgrunner(self)
    pkgr = self.pkgr
  
    if hasattr(pkgr, "run") and callable(getattr(pkgr, "run")):
      pkgr.run()

    # -------------- Touch events --------------
    self.tPT = PollThread(0.02, self.getTouchEvents, "tTouchEvents")
    self.tPT.start()


  def getTouchEvents(self):
    self.gui.DoGUI()

  def OnGuiMessage(self, msg):
    pkgr = self.pkgr
    if hasattr(pkgr, "OnGuiMessage") and callable(getattr(pkgr, "OnGuiMessage")):
      pkgr.OnGuiMessage(msg)
    else:
      print('Fallback Info - Message received: ', msg.type, " ", msg.m_evt_type)

  def OnCanMessage(self, id, byte_str, len):
    pkgr = self.pkgr
    if hasattr(pkgr, "OnCanMessage") and callable(getattr(pkgr, "OnCanMessage")):
      pkgr.OnCanMessage(id, byte_str, len)
    else:
      print(f"Fallback Info - Can Message received: id={id}, data={byte_str}, len={len}")

mgr=mpkgmanager(sys.argv[1])
